#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
# SPDX-License-Identifier: GPL-3.0
#
# GNU Radio Python Flow Graph
# Title: Pluto_frequency_shifter
# GNU Radio version: 3.10.5.1

from packaging.version import Version as StrictVersion

if __name__ == '__main__':
    import ctypes
    import sys
    if sys.platform.startswith('linux'):
        try:
            x11 = ctypes.cdll.LoadLibrary('libX11.so')
            x11.XInitThreads()
        except:
            print("Warning: failed to XInitThreads()")

from PyQt5 import Qt
from PyQt5.QtCore import QObject, pyqtSlot
from gnuradio import qtgui
from gnuradio.filter import firdes
import sip
from gnuradio import analog
from gnuradio import filter
from gnuradio import gr
from gnuradio.fft import window
import sys
import signal
from argparse import ArgumentParser
from gnuradio.eng_arg import eng_float, intx
from gnuradio import eng_notation
from gnuradio import iio
from gnuradio.qtgui import Range, RangeWidget
from PyQt5 import QtCore



from gnuradio import qtgui

class Pluto_frequency_shifter(gr.top_block, Qt.QWidget):

    def __init__(self):
        gr.top_block.__init__(self, "Pluto_frequency_shifter", catch_exceptions=True)
        Qt.QWidget.__init__(self)
        self.setWindowTitle("Pluto_frequency_shifter")
        qtgui.util.check_set_qss()
        try:
            self.setWindowIcon(Qt.QIcon.fromTheme('gnuradio-grc'))
        except:
            pass
        self.top_scroll_layout = Qt.QVBoxLayout()
        self.setLayout(self.top_scroll_layout)
        self.top_scroll = Qt.QScrollArea()
        self.top_scroll.setFrameStyle(Qt.QFrame.NoFrame)
        self.top_scroll_layout.addWidget(self.top_scroll)
        self.top_scroll.setWidgetResizable(True)
        self.top_widget = Qt.QWidget()
        self.top_scroll.setWidget(self.top_widget)
        self.top_layout = Qt.QVBoxLayout(self.top_widget)
        self.top_grid_layout = Qt.QGridLayout()
        self.top_layout.addLayout(self.top_grid_layout)

        self.settings = Qt.QSettings("GNU Radio", "Pluto_frequency_shifter")

        try:
            if StrictVersion(Qt.qVersion()) < StrictVersion("5.0.0"):
                self.restoreGeometry(self.settings.value("geometry").toByteArray())
            else:
                self.restoreGeometry(self.settings.value("geometry"))
        except:
            pass

        ##################################################
        # Variables
        ##################################################
        self.tx_freq = tx_freq = 2322500000
        self.tx_drive = tx_drive = 94
        self.samp_rate = samp_rate = 2250000
        self.rx_freq_offset = rx_freq_offset = 0
        self.rx_freq_coarse = rx_freq_coarse = 3402500000
        self.rx_bandwidth = rx_bandwidth = 2.25
        self.rrx_gain = rrx_gain = 70
        self.dc_spike_left = dc_spike_left = 0
        self.dc___spike_left = dc___spike_left = 562500

        ##################################################
        # Blocks
        ##################################################

        # Create the options list
        self._tx_freq_options = [146500000, 437000000, 1298500000, 2322500000]
        # Create the labels list
        self._tx_freq_labels = ['2m (146.5MHz)', '70cm (437MHz)', '23cm (1298.5MHz)', '13cm (2322.5MHz)']
        # Create the combo box
        # Create the radio buttons
        self._tx_freq_group_box = Qt.QGroupBox("F2 TX Frequency to Picotuner (MHz)" + ": ")
        self._tx_freq_box = Qt.QHBoxLayout()
        class variable_chooser_button_group(Qt.QButtonGroup):
            def __init__(self, parent=None):
                Qt.QButtonGroup.__init__(self, parent)
            @pyqtSlot(int)
            def updateButtonChecked(self, button_id):
                self.button(button_id).setChecked(True)
        self._tx_freq_button_group = variable_chooser_button_group()
        self._tx_freq_group_box.setLayout(self._tx_freq_box)
        for i, _label in enumerate(self._tx_freq_labels):
            radio_button = Qt.QRadioButton(_label)
            self._tx_freq_box.addWidget(radio_button)
            self._tx_freq_button_group.addButton(radio_button, i)
        self._tx_freq_callback = lambda i: Qt.QMetaObject.invokeMethod(self._tx_freq_button_group, "updateButtonChecked", Qt.Q_ARG("int", self._tx_freq_options.index(i)))
        self._tx_freq_callback(self.tx_freq)
        self._tx_freq_button_group.buttonClicked[int].connect(
            lambda i: self.set_tx_freq(self._tx_freq_options[i]))
        self.top_layout.addWidget(self._tx_freq_group_box)
        self._tx_drive_range = Range(1, 100, 1, 94, 200)
        self._tx_drive_win = RangeWidget(self._tx_drive_range, self.set_tx_drive, "Pluto TX Drive (1  to 100)", "counter_slider", float, QtCore.Qt.Horizontal)
        self.top_layout.addWidget(self._tx_drive_win)
        self._rx_freq_offset_range = Range(-6000, 6000, 50000000, 0, 200)
        self._rx_freq_offset_win = RangeWidget(self._rx_freq_offset_range, self.set_rx_freq_offset, "F1 Frequency Offset (MHz) Step (50kHz)", "counter_slider", float, QtCore.Qt.Horizontal)
        self.top_layout.addWidget(self._rx_freq_offset_win)
        # Create the options list
        self._rx_freq_coarse_options = [51000000, 71000000, 3402500000, 5762500000, 0]
        # Create the labels list
        self._rx_freq_coarse_labels = ['6m (51.0MHz)', '4m (71.0MHz)', '9cm (3402.5MHz)', '6cm (5762.5MHz)', '0Hz Offset']
        # Create the combo box
        self._rx_freq_coarse_tool_bar = Qt.QToolBar(self)
        self._rx_freq_coarse_tool_bar.addWidget(Qt.QLabel("F1 RX Frequency (MHz)" + ": "))
        self._rx_freq_coarse_combo_box = Qt.QComboBox()
        self._rx_freq_coarse_tool_bar.addWidget(self._rx_freq_coarse_combo_box)
        for _label in self._rx_freq_coarse_labels: self._rx_freq_coarse_combo_box.addItem(_label)
        self._rx_freq_coarse_callback = lambda i: Qt.QMetaObject.invokeMethod(self._rx_freq_coarse_combo_box, "setCurrentIndex", Qt.Q_ARG("int", self._rx_freq_coarse_options.index(i)))
        self._rx_freq_coarse_callback(self.rx_freq_coarse)
        self._rx_freq_coarse_combo_box.currentIndexChanged.connect(
            lambda i: self.set_rx_freq_coarse(self._rx_freq_coarse_options[i]))
        # Create the radio buttons
        self.top_layout.addWidget(self._rx_freq_coarse_tool_bar)
        self._rx_bandwidth_range = Range(750m, 10, 250, 2.25, 200)
        self._rx_bandwidth_win = RangeWidget(self._rx_bandwidth_range, self.set_rx_bandwidth, "RX Bandwidth (MHz) Step (250KHz)", "counter_slider", float, QtCore.Qt.Horizontal)
        self.top_layout.addWidget(self._rx_bandwidth_win)
        self._rrx_gain_range = Range(1, 75, 1, 70, 200)
        self._rrx_gain_win = RangeWidget(self._rrx_gain_range, self.set_rrx_gain, "Pluto     RX Gain (1 to 75)", "counter_slider", float, QtCore.Qt.Horizontal)
        self.top_layout.addWidget(self._rrx_gain_win)
        self.qtgui_freq_sink_x_0 = qtgui.freq_sink_c(
            1024, #size
            window.WIN_BLACKMAN_hARRIS, #wintype
            2.25G, #fc
            2.25M, #bw
            'F1 RX Spectrum', #name
            1,
            None # parent
        )
        self.qtgui_freq_sink_x_0.set_update_time(0.10)
        self.qtgui_freq_sink_x_0.set_y_axis((-140), 10)
        self.qtgui_freq_sink_x_0.set_y_label('Relative Gain', 'dB')
        self.qtgui_freq_sink_x_0.set_trigger_mode(qtgui.TRIG_MODE_FREE, 0.0, 0, "")
        self.qtgui_freq_sink_x_0.enable_autoscale(False)
        self.qtgui_freq_sink_x_0.enable_grid(False)
        self.qtgui_freq_sink_x_0.set_fft_average(1.0)
        self.qtgui_freq_sink_x_0.enable_axis_labels(True)
        self.qtgui_freq_sink_x_0.enable_control_panel(False)
        self.qtgui_freq_sink_x_0.set_fft_window_normalized(False)



        labels = ['', '', '', '', '',
            '', '', '', '', '']
        widths = [1, 1, 1, 1, 1,
            1, 1, 1, 1, 1]
        colors = ["blue", "red", "green", "black", "cyan",
            "magenta", "yellow", "dark red", "dark green", "dark blue"]
        alphas = [1.0, 1.0, 1.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0, 1.0]

        for i in range(1):
            if len(labels[i]) == 0:
                self.qtgui_freq_sink_x_0.set_line_label(i, "Data {0}".format(i))
            else:
                self.qtgui_freq_sink_x_0.set_line_label(i, labels[i])
            self.qtgui_freq_sink_x_0.set_line_width(i, widths[i])
            self.qtgui_freq_sink_x_0.set_line_color(i, colors[i])
            self.qtgui_freq_sink_x_0.set_line_alpha(i, alphas[i])

        self._qtgui_freq_sink_x_0_win = sip.wrapinstance(self.qtgui_freq_sink_x_0.qwidget(), Qt.QWidget)
        self.top_layout.addWidget(self._qtgui_freq_sink_x_0_win)
        self.iio_pluto_source_0 = iio.fmcomms2_source_fc32('192.168.2.1' if '192.168.2.1' else iio.get_pluto_uri(), [True, True], 32768)
        self.iio_pluto_source_0.set_len_tag_key('packet_len')
        self.iio_pluto_source_0.set_frequency(3402500000)
        self.iio_pluto_source_0.set_samplerate(2250000)
        self.iio_pluto_source_0.set_gain_mode(0, 'slow_attack')
        self.iio_pluto_source_0.set_gain(0, 64)
        self.iio_pluto_source_0.set_quadrature(True)
        self.iio_pluto_source_0.set_rfdc(True)
        self.iio_pluto_source_0.set_bbdc(True)
        self.iio_pluto_source_0.set_filter_params('Auto', '', 0, 0)
        self.iio_pluto_sink_0 = iio.fmcomms2_sink_fc32('192.168.2.1' if '192.168.2.1' else iio.get_pluto_uri(), [True, True], 32768, False)
        self.iio_pluto_sink_0.set_len_tag_key('')
        self.iio_pluto_sink_0.set_bandwidth(20000000)
        self.iio_pluto_sink_0.set_frequency(2322500000)
        self.iio_pluto_sink_0.set_samplerate(2250000)
        self.iio_pluto_sink_0.set_attenuation(0, 94)
        self.iio_pluto_sink_0.set_filter_params('Auto', '', 0, 0)
        self.dc_blocker_xx_0 = filter.dc_blocker_cc(128, True)
        self.analog_agc_xx_0 = analog.agc_cc((1e-4), 1.0, 1.0)
        self.analog_agc_xx_0.set_max_gain(65536)


        ##################################################
        # Connections
        ##################################################
        self.connect((self.analog_agc_xx_0, 0), (self.iio_pluto_sink_0, 0))
        self.connect((self.analog_agc_xx_0, 0), (self.qtgui_freq_sink_x_0, 0))
        self.connect((self.dc_blocker_xx_0, 0), (self.analog_agc_xx_0, 0))
        self.connect((self.iio_pluto_source_0, 0), (self.dc_blocker_xx_0, 0))


    def closeEvent(self, event):
        self.settings = Qt.QSettings("GNU Radio", "Pluto_frequency_shifter")
        self.settings.setValue("geometry", self.saveGeometry())
        self.stop()
        self.wait()

        event.accept()

    def get_tx_freq(self):
        return self.tx_freq

    def set_tx_freq(self, tx_freq):
        self.tx_freq = tx_freq
        self._tx_freq_callback(self.tx_freq)

    def get_tx_drive(self):
        return self.tx_drive

    def set_tx_drive(self, tx_drive):
        self.tx_drive = tx_drive

    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate

    def get_rx_freq_offset(self):
        return self.rx_freq_offset

    def set_rx_freq_offset(self, rx_freq_offset):
        self.rx_freq_offset = rx_freq_offset

    def get_rx_freq_coarse(self):
        return self.rx_freq_coarse

    def set_rx_freq_coarse(self, rx_freq_coarse):
        self.rx_freq_coarse = rx_freq_coarse
        self._rx_freq_coarse_callback(self.rx_freq_coarse)

    def get_rx_bandwidth(self):
        return self.rx_bandwidth

    def set_rx_bandwidth(self, rx_bandwidth):
        self.rx_bandwidth = rx_bandwidth

    def get_rrx_gain(self):
        return self.rrx_gain

    def set_rrx_gain(self, rrx_gain):
        self.rrx_gain = rrx_gain

    def get_dc_spike_left(self):
        return self.dc_spike_left

    def set_dc_spike_left(self, dc_spike_left):
        self.dc_spike_left = dc_spike_left

    def get_dc___spike_left(self):
        return self.dc___spike_left

    def set_dc___spike_left(self, dc___spike_left):
        self.dc___spike_left = dc___spike_left




def main(top_block_cls=Pluto_frequency_shifter, options=None):

    if StrictVersion("4.5.0") <= StrictVersion(Qt.qVersion()) < StrictVersion("5.0.0"):
        style = gr.prefs().get_string('qtgui', 'style', 'raster')
        Qt.QApplication.setGraphicsSystem(style)
    qapp = Qt.QApplication(sys.argv)

    tb = top_block_cls()

    tb.start()

    tb.show()

    def sig_handler(sig=None, frame=None):
        tb.stop()
        tb.wait()

        Qt.QApplication.quit()

    signal.signal(signal.SIGINT, sig_handler)
    signal.signal(signal.SIGTERM, sig_handler)

    timer = Qt.QTimer()
    timer.start(500)
    timer.timeout.connect(lambda: None)

    qapp.exec_()

if __name__ == '__main__':
    main()
